@@ -21,6 +21,102 @@ local ipairs = ipairs
 
 -- this module provide methods to generate snippets which will be used in the nginx.conf template
 local _M = {}
+local conf_server_tpl = [[
+upstream apisix_conf_backend {
+    server 0.0.0.0:80;
+    balancer_by_lua_block {
+        local conf_server = require("apisix.conf_server")
+        conf_server.balancer()
+    }
+    keepalive 320;
+    keepalive_requests 1000;
+    keepalive_timeout 60s;
+}
+
+{% if trusted_ca_cert then %}
+lua_ssl_trusted_certificate {* trusted_ca_cert *};
+{% end %}
+
+server {
+    {% if control_plane then %}
+    {% if directive_prefix == "grpc" then %}
+    listen {* control_plane.listen *} ssl http2;
+    {% else %}
+    listen {* control_plane.listen *} ssl;
+    {% end %}
+    ssl_certificate {* control_plane.cert *};
+    ssl_certificate_key {* control_plane.cert_key *};
+
+    {% if control_plane.client_ca_cert then %}
+    ssl_verify_client on;
+    ssl_client_certificate {* control_plane.client_ca_cert *};
+    {% end %}
+
+    {% else %}
+    {% if directive_prefix == "grpc" then %}
+    listen unix:{* home *}/conf/config_listen.sock http2;
+    {% else %}
+    listen unix:{* home *}/conf/config_listen.sock;
+    {% end %}
+    {% end %}
+
+    access_log off;
+
+    set $upstream_host '';
+
+    access_by_lua_block {
+        local conf_server = require("apisix.conf_server")
+        conf_server.access()
+    }
+
+    location / {
+        {% if enable_https then %}
+        {* directive_prefix *}_pass {* scheme_name *}s://apisix_conf_backend;
+        {* directive_prefix *}_ssl_protocols TLSv1.2 TLSv1.3;
+        {* directive_prefix *}_ssl_server_name on;
+
+        {% if etcd_tls_verify then %}
+        {* directive_prefix *}_ssl_verify on;
+        {* directive_prefix *}_ssl_trusted_certificate {* ssl_trusted_certificate *};
+        {% end %}
+
+        {% if sni then %}
+        {* directive_prefix *}_ssl_name {* sni *};
+        {% else %}
+        {* directive_prefix *}_ssl_name $upstream_host;
+        {% end %}
+
+        {% if client_cert then %}
+        {* directive_prefix *}_ssl_certificate {* client_cert *};
+        {* directive_prefix *}_ssl_certificate_key {* client_cert_key *};
+        {% end %}
+
+        {% else %}
+        {* directive_prefix *}_pass {* scheme_name *}://apisix_conf_backend;
+        {% end %}
+
+        {% if scheme_name == "http"  then %}
+        proxy_http_version 1.1;
+        proxy_set_header Connection "";
+        {% end %}
+
+        {* directive_prefix *}_set_header Host $upstream_host;
+        {* directive_prefix *}_next_upstream error timeout non_idempotent
+            http_500 http_502 http_503 http_504;
+    }
+
+    log_by_lua_block {
+        local conf_server = require("apisix.conf_server")
+        conf_server.log()
+    }
+}
+]]
+
+
+local function is_grpc_used(env, etcd)
+    local is_grpc_available = env.use_apisix_base
+    return is_grpc_available and etcd.use_grpc
+end
 
 
 function _M.generate_conf_server(env, conf)
@@ -69,87 +165,13 @@ function _M.generate_conf_server(env, conf)
         end
     end
 
-    local conf_render = template.compile([[
-    upstream apisix_conf_backend {
-        server 0.0.0.0:80;
-        balancer_by_lua_block {
-            local conf_server = require("apisix.conf_server")
-            conf_server.balancer()
-        }
-    }
-
-    {% if trusted_ca_cert then %}
-    lua_ssl_trusted_certificate {* trusted_ca_cert *};
-    {% end %}
-
-    server {
-        {% if control_plane then %}
-        listen {* control_plane.listen *} ssl;
-        ssl_certificate {* control_plane.cert *};
-        ssl_certificate_key {* control_plane.cert_key *};
-
-        {% if control_plane.client_ca_cert then %}
-        ssl_verify_client on;
-        ssl_client_certificate {* control_plane.client_ca_cert *};
-        {% end %}
-
-        {% else %}
-        listen unix:{* home *}/conf/config_listen.sock;
-        {% end %}
-
-        access_log off;
-
-        set $upstream_host '';
-
-        access_by_lua_block {
-            local conf_server = require("apisix.conf_server")
-            conf_server.access()
-        }
-
-        location / {
-            {% if enable_https then %}
-            proxy_pass https://apisix_conf_backend;
-            proxy_ssl_protocols TLSv1.2 TLSv1.3;
-            proxy_ssl_server_name on;
-
-            {% if etcd_tls_verify then %}
-            proxy_ssl_verify on;
-            proxy_ssl_trusted_certificate {* ssl_trusted_certificate *};
-            {% end %}
-
-            {% if sni then %}
-            proxy_ssl_name {* sni *};
-            {% else %}
-            proxy_ssl_name $upstream_host;
-            {% end %}
-
-            {% if client_cert then %}
-            proxy_ssl_certificate {* client_cert *};
-            proxy_ssl_certificate_key {* client_cert_key *};
-            {% end %}
-
-            {% else %}
-            proxy_pass http://apisix_conf_backend;
-            {% end %}
-
-            proxy_http_version 1.1;
-            proxy_set_header Connection "";
-            proxy_set_header Host $upstream_host;
-            proxy_next_upstream error timeout non_idempotent http_500 http_502 http_503 http_504;
-        }
-
-        log_by_lua_block {
-            local conf_server = require("apisix.conf_server")
-            conf_server.log()
-        }
-    }
-    ]])
-
+    local conf_render = template.compile(conf_server_tpl)
     local tls = etcd.tls
     local client_cert
     local client_cert_key
     local ssl_trusted_certificate
     local etcd_tls_verify
+    local use_grpc = is_grpc_used(env, etcd)
     if tls then
         if tls.cert then
             client_cert = pl_path.abspath(tls.cert)
@@ -175,6 +197,8 @@ function _M.generate_conf_server(env, conf)
         trusted_ca_cert = trusted_ca_cert,
         etcd_tls_verify = etcd_tls_verify,
         ssl_trusted_certificate = ssl_trusted_certificate,
+        scheme_name = use_grpc and "grpc" or "http",
+        directive_prefix = use_grpc and "grpc" or "proxy",
     })
 end
 
