@@ -17,15 +17,10 @@
 local core = require("apisix.core")
 local ngx = ngx
 local ngx_re = require("ngx.re")
-local ipairs = ipairs
 local consumer = require("apisix.consumer")
-
 local lrucache = core.lrucache.new({
     ttl = 300, count = 512
 })
-local consumers_lrucache = core.lrucache.new({
-    type = "plugin",
-})
 
 local schema = {
     type = "object",
@@ -45,11 +40,13 @@ local consumer_schema = {
         username = { type = "string" },
         password = { type = "string" },
     },
+    encrypt_fields = {"password"},
     required = {"username", "password"},
 }
 
 local plugin_name = "basic-auth"
 
+
 local _M = {
     version = 0.1,
     priority = 2520,
@@ -122,22 +119,6 @@ local function extract_auth_header(authorization)
 
 end
 
-local create_consume_cache
-do
-    local consumer_names = {}
-
-    function create_consume_cache(consumers)
-        core.table.clear(consumer_names)
-
-        for _, cur_consumer in ipairs(consumers.nodes) do
-            core.log.info("consumer node: ",
-                          core.json.delay_encode(cur_consumer))
-            consumer_names[cur_consumer.auth_conf.username] = cur_consumer
-        end
-
-        return consumer_names
-    end
-end
 
 function _M.rewrite(conf, ctx)
     core.log.info("plugin access phase, conf: ", core.json.delay_encode(conf))
@@ -161,9 +142,7 @@ function _M.rewrite(conf, ctx)
         return 401, { message = "Missing related consumer" }
     end
 
-    local consumers = consumers_lrucache("consumers_key",
-        consumer_conf.conf_version,
-        create_consume_cache, consumer_conf)
+    local consumers = consumer.consumers_kv(plugin_name, consumer_conf, "username")
 
     -- 3. check user exists
     local cur_consumer = consumers[username]
