@@ -128,11 +128,12 @@ local function scan_log_folder(log_file_name)
 
     local log_dir, _ = get_log_path_info(log_file_name)
 
+    local compression_log_type = log_file_name .. COMPRESSION_FILE_SUFFIX
     for file in lfs.dir(log_dir) do
         local n = get_last_index(file, "__")
         if n ~= nil then
             local log_type = file:sub(n + 2)
-            if log_type == log_file_name then
+            if log_type == log_file_name or log_type == compression_log_type then
                 core.table.insert(t, file)
             end
         end
